@@ -159,6 +159,7 @@ class FlawlessService(object):
     self.persistent_dict_cls = persistent_dict_cls
     self.time_func = time_func
     self.thread_cls = thread_cls
+    self.number_of_git_blames_running = 0
 
     self.building_blocks = self._parse_whitelist_file("building_blocks", BuildingBlock)
     self.third_party_whitelist = self._parse_whitelist_file("third_party_whitelist",
@@ -284,6 +285,9 @@ class FlawlessService(object):
     '''Given an email address, check the email_remapping table to see if the email
     should be sent to a different address. This function also handles overriding
     the email domain if ignore_vcs_email_domain is set or the domain was missing'''
+    if not email or "@" not in email:
+      return None
+
     if email in self.email_remapping:
       return self.email_remapping[email]
     prefix, domain = email.split("@", 2)
@@ -340,19 +344,19 @@ class FlawlessService(object):
 
   def _format_traceback(self, request, append_locals=True, show_full_stack=False,
                         linebreak="<br />", spacer="&nbsp;", start_bold="<strong>",
-                        end_bold="</strong>"):
+                        end_bold="</strong>", escape_func=cgi.escape):
     # Traceback
     parts = []
     parts.append("{b}Traceback (most recent call last):{xb}".format(b=start_bold, xb=end_bold))
     formatted_stack = [
       '{sp}{sp}File "{filename}", line {line}, in {function}{lb}{sp}{sp}{sp}{sp}{code}'.format(
         sp=spacer, lb=linebreak, filename=l.filename, line=l.line_number,
-        function=l.function_name, code=l.text,
+        function=l.function_name, code=escape_func(l.text),
       )
       for l in request.traceback
     ]
     parts.extend(formatted_stack)
-    parts.append(request.exception_message)
+    parts.append(escape_func(request.exception_message))
 
     # Frame Locals
     parts.append(linebreak * 2 + "{b}Stack Frame:{xb}".format(b=start_bold, xb=end_bold))
@@ -365,7 +369,7 @@ class FlawlessService(object):
         b=start_bold, xb=end_bold,
       )
       local_vals = ['{sp}{sp}{sp}{sp}{name}={value}'.format(
-                        sp=spacer, name=name, value=value.decode("UTF-8", "replace"))
+                        sp=spacer, name=name, value=escape_func(value.decode("UTF-8", "replace")))
                     for name, value in sorted(l.frame_locals.items())]
       parts.append(line_info)
       parts.extend(local_vals or [spacer * 4 + "No variables in this frame"])
@@ -373,7 +377,9 @@ class FlawlessService(object):
     # Additional Information
     if request.additional_info:
       parts.append(linebreak * 2 + "{b}Additional Information:{xb}".format(b=start_bold, xb=end_bold))
-      parts.append(request.additional_info.decode("UTF-8", "replace").replace("\n", linebreak))
+      parts.append(
+          escape_func(request.additional_info.decode("UTF-8", "replace")).replace("\n", linebreak)
+      )
 
     return linebreak.join(parts)
 
@@ -416,21 +422,40 @@ class FlawlessService(object):
     # If this error hasn't been reported before, then find the dev responsible
     err_info = None
     if key not in self.errors_seen:
-      email, last_touched_ts = self.repository.blame(key.filename, key.line_number)
-      if email:
-        cur_time = self._convert_epoch_ms(datetime.datetime).strftime("%Y-%m-%d %H:%M:%S")
-        mod_time = self._convert_epoch_ms(datetime.datetime, epoch_ms=last_touched_ts * 1000)
-        mod_time = mod_time.strftime("%Y-%m-%d %H:%M:%S")
-        known_entry = self._get_entry(blamed_entry, self.known_errors)
-        err_info = api.ErrorInfo(error_count=1,
-                                 developer_email=self._get_email(email),
-                                 date=mod_time,
-                                 email_sent=False,
-                                 last_occurrence=cur_time,
-                                 is_known_error=bool(known_entry),
-                                 last_error_data=request)
-        self.errors_seen[key] = err_info
-        log.info("Error %s caused by %s on %s" % (str(key), email, mod_time))
+      # If flawless is being flooded wih errors, limit the number of git blames so the
+      # service doesn't fall over. We don't use a thread safe counter, because 10
+      # git blames is just a soft limit
+      if self.number_of_git_blames_running > config.max_concurrent_git_blames:
+        log.error("Unable to process %s because %d git blames already running" %
+                  (str(key), self.number_of_git_blames_running))
+        return
+      try:
+        self.number_of_git_blames_running += 1
+        email, last_touched_ts = self.repository.blame(key.filename, key.line_number)
+      finally:
+        self.number_of_git_blames_running -= 1
+      dev_email = self._get_email(email)
+      last_touched_ts = last_touched_ts or 0
+
+      cur_time = self._convert_epoch_ms(datetime.datetime).strftime("%Y-%m-%d %H:%M:%S")
+      mod_time = self._convert_epoch_ms(datetime.datetime, epoch_ms=last_touched_ts * 1000)
+      mod_time = mod_time.strftime("%Y-%m-%d %H:%M:%S")
+      known_entry = self._get_entry(blamed_entry, self.known_errors)
+      err_info = api.ErrorInfo(error_count=1,
+                               developer_email=dev_email or "unknown",
+                               date=mod_time,
+                               email_sent=False,
+                               last_occurrence=cur_time,
+                               is_known_error=bool(known_entry),
+                               last_error_data=request)
+      self.errors_seen[key] = err_info
+      log.info("Error %s caused by %s on %s" % (str(key), dev_email, mod_time))
+
+      if not dev_email:
+        self._handle_flawless_issue("Unable to do blame for %s. You may want to consider setting "
+                                    "only_blame_filepaths_matching in your flawless.cfg " % str(key))
+        err_info.email_sent = True
+        return
     # If we've already seen this error then update the error count
     elif key in self.errors_seen:
       err_info = self.errors_seen[key]
@@ -440,11 +465,6 @@ class FlawlessService(object):
       err_info.last_occurrence = cur_dt.strftime("%Y-%m-%d %H:%M:%S")
       self.errors_seen[key] = err_info
 
-    if not err_info:
-      self._handle_flawless_issue("Unable to do blame for %s. You may want to consider setting "
-                                  "only_blame_filepaths_matching in your flawless.cfg " % str(key))
-      return
-
     # Figure out if we should send an email or not
     send_email = False
     known_entry = None
@@ -475,7 +495,9 @@ class FlawlessService(object):
     # Send email if applicable
     if send_email:
       email_body = []
-      email_recipients.append(self._get_email(err_info.developer_email))
+      dev_email = self._get_email(err_info.developer_email)
+      if dev_email:
+        email_recipients.append(dev_email)
 
       # Add additional recipients that have registered for this error
       if blamed_entry.filename in self.watch_only_if_blamed:
@@ -656,8 +678,10 @@ class FlawlessService(object):
     class_map = dict(known_errors=KnownError, building_blocks=BuildingBlock,
                      third_party_whitelist=ThirdPartyWhitelistEntry)
     cls = class_map[params['type']]
+    init_args = inspect.getargspec(CodeIdentifierBaseClass.__init__ if cls == KnownError
+                                   else cls.__init__).args
 
-    whitelist_attrs = [s for s in inspect.getargspec(cls.__init__).args if s != 'self']
+    whitelist_attrs = [s for s in init_args if s != 'self']
     new_entry = cls(**dict((k,params.get(k)) for k in whitelist_attrs))
     filename = os.path.join(config.config_dir_path, params['type'])
 
